import pprint
import time
import random
import logging

from smtplib import SMTP

logger = logging.getLogger(__name__)


def batch_filter_entrypoint(batch_item={}, config={}):
    data = batch_item.get("data", "data-field-missing")
    logger.info(f"MAIL TOKEN CONFIRM RUNNING WITH {data} ######################")
    domain = config.get("email-domain")
    user = config.get("email-user")
    password = config.get("email-password")
    sender = f"{user}@{domain}"
    receivers = ["test-spam@octomy.org"]
    message_template = """From: First Kiss <{0}>
To: <{1}>
Subject: Hello 
This is a test"""
    with SMTP("postfix") as smtp:
        smtp.login(sender, password)
        for receiver in receivers:
            logger.info(f"Sending email from{sender} to {receiver}")
            smtp.sendmail(sender, receiver, message_template.format(sender, receiver))



import pprint
import time
import random
import logging
import re
import json
import requests
from validate_email import validate_email
from urllib.parse import urlparse
import fk.utils
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

logger = logging.getLogger(__name__)


def verify_website(url):
    return bool(requests.get(url))


def shopify_domain_for_url(url):
    myshopify_domain = None
    rex = "([0-9a-z\-_]{1,20}\.myshopify\.com)"
    # If the url is on the myshopify form already, go at it directly
    if not url:
        return None, "No url"
    if ".myshopify.com" in url:
        m = re.search(rex, url)
        myshopify_domain = m.group(0)
    elif url.startsWith("http://") or url.startsWith("https://"):
        parts = urlparse(url)
        canonical_url = f"{parts['scheme']}://{parts['hostname']}"
        r = requests.get(canonical_url)
        if r:
            m = re.search(rex, r.body)
            myshopify_domain = m.group(0)
        else:
            return None, f"Could not fetch canonical URL: '{canonical_url}' for '{url}'"
    if myshopify_domain:
        if verify_website(f"https://{url}"):
            return myshopify_domain, None
        else:
            return None, f"Invalid myshopify_domain found: {myshopify_domain}"
    else:
        return None, f"No myshopify_domain found: {myshopify_domain}"
    return None, "Unknown error"


def prepare_email(sender, receiver, subject, body, body_html):
    # Create message container - the correct MIME type is multipart/alternative here!
    msg = MIMEMultipart("alternative")
    msg["subject"] = subject
    msg["To"] = receiver
    msg["From"] = sender
    msg.preamble = body
    msg.attach(MIMEText(body_html, "html"))
    return msg.as_string()


def batch_filter_entrypoint(batch_item={}, config={}):
    data_str = batch_item.get("data", "")
    # logger.info(f"SHOPIFY BETA INGEST RUNNING WITH {data_str} ######################")
    data = {}
    try:
        data = json.loads(data_str)
    except Exception as e:
        return None, f"Could not parse json: {data_str}"
    receiver = data.get("email")
    shop_url = data.get("shop_url")
    myshopify_domain, myshopify_error = shopify_domain_for_url(shop_url)
    if not myshopify_domain:
        return None, myshopify_error or "Unknown error"
    if not validate_email(receiver, check_mx=True, verify=True):
        return None, f"Email invalid: '{receiver}'"
    domain = config.get("email-domain")
    user = config.get("email-user")
    sender = f"{user}@{domain}"
    password = config.get("email-password")
    nonce = fk.utils.random_str(32)
    try:
        with smtplib.SMTP("postfix") as smtp:
            body_template = """This is an autoamted  message to confirm that your FirstKiss Salespop BETA submission was accepted.\r\n\r\n
To start installation, please click the link:\r\n
https://{myshopify_domain}/admin?nonce={nonce}
"""
            body_html_template = """
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
<title>FirstKiss Salespop BETA submission</title>
<style type="text/css" media="screen">
*{{
    font-family:sans;
}}
</style>
<h1>Congratulations!</h1>
<p>This is autoamted message is a confirmation that your FirstKiss Salespop BETA submission was accepted!\n
To start installation, please click the link:</p>
<br/>
<a href="https://{myshopify_domain}/admin?nonce={nonce}">install FirstKiss Salespop</a>
"""
            try:
                # logger.info(f"Logging into '{sender}' with '{password}'")
                smtp.login(sender, password)
            except Exception as e:
                return None, f"Error logging into email server: {e}"
            try:
                body = body_template.format(sender=sender, receiver=receiver, myshopify_domain=myshopify_domain, nonce=nonce)
                body_html = body_html_template.format(sender=sender, receiver=receiver, myshopify_domain=myshopify_domain, nonce=nonce)
                subject = f"BETA confirmation for {myshopify_domain}"
                message = prepare_email(sender=sender, receiver=receiver, subject=subject, body=body, body_html=body_html)
                # logger.info(f"Sending message '{message}' from '{sender}' to '{receiver}'")
                smtp.sendmail(sender, receiver, message)
            except Exception as e:
                return None, f"Error sending email: {e}"
    except Exception as e:
        return None, f"Error with email sending: {e}"
    return None, None



import logging

from fk.api.shopify import ShopifySessionManager
from fk.api.shopify.db import Database, get_flask_shopify_db

import json

logger = logging.getLogger(__name__)


def batch_filter_entrypoint(batch_item={}, config={}):
    # logger.info(f"SHOPIFY ORDERS FETCH RUNNING WITH {batch_item} ######################")
    data = json.loads(batch_item.get("data", {}))
    if not data:
        return None, "No data"
    shop_domain = data.get("shop_domain")
    if not shop_domain:
        return None, "No shop domain"
    topic = data.get("topic")
    if not order_id:
        return None, "No topic"
    shop_domain = shop_domain.strip()
    logger.info(f"Effectively aknowleging webhook of topic {topic} for {shop_domain}")
    return None, None



import logging

from fk.api.shopify import ShopifySessionManager
from fk.api.shopify.db import Database, get_flask_shopify_db

# from fk.api.shopify.Database import *
import pprint
import time
import random
import json

logger = logging.getLogger(__name__)

# fmt:off

product_variant_fields = ["id",
    "product_id",
    "title",
    "price",
    "sku",
    "position",
    "inventory_policy",
    "compare_at_price",
    "fulfillment_service",
    "inventory_management",
    "option1",
    "option2",
    "option3",
    "taxable",
    "barcode",
    "grams",
    "image_id",
    "weight",
    "weight_unit",
    "inventory_item_id",
    "inventory_quantity",
    "old_inventory_quantity",
    "requires_shipping",
    "admin_graphql_api_id",
    "json"]
# fmt:on


product_variant_dict_default = {}

for f in product_variant_fields:
    product_variant_dict_default[f] = None


def batch_filter_entrypoint(batch_item={}, config={}):
    # logger.info(f"SHOPIFY PRODUCTS FETCH RUNNING WITH {batch_item} ######################")
    data = json.loads(batch_item.get("data", {}))
    if not data:
        return None, "No data"
    shop_domain = data.get("shop_domain")
    if not shop_domain:
        return None, "No shop url"
    product_variant_id = data.get("product_variant_id")
    if not product_variant_id:
        return None, "No product_variant_id"
    product_id = data.get("product_id")
    if not product_id:
        return None, "No product_id"
    shop_domain = shop_domain.strip()
    db = Database(config)  # No flask context: get_flask_shopify_db(config)
    shop = db.get_shop(shop_domain)
    if not shop:
        return None, "No shop"
    token = shop.get("offline_access_token", None)
    if not token:
        return None, "No token"
    sm = ShopifySessionManager(config=config, db=db, shop_domain=shop_domain, token=token)
    product_variant = sm.get_product_variant_by_id(product_id, product_variant_id)
    # logger.warning(pprint.pformat(product_variant))
    if not product_variant:
        return None, "No product_variant"
    product_variant_dict_raw = product_variant.to_dict()
    product_variant_dict = {**product_variant_dict_default, **product_variant_dict_raw}
    if product_variant_dict["id"] != product_variant_id:
        return None, f"Returned product_variant {product_variant_dict['id']} did not match initial id {product_variant_id}"
    # Augment it with some needed extras
    product_variant_dict["myshopify_name"] = shop_domain
    product_variant_dict["json"] = json.dumps(product_variant_dict, indent=3, sort_keys=True)
    product_variant_id = db.upsert_product_variant(product_variant_dict)
    if not product_variant_id:
        return None, "Could not save product_variant"
    return None, None



import logging

from fk.api.shopify import ShopifySessionManager
from fk.api.shopify.db import Database, get_flask_shopify_db
import fk.batch.BatchProcessor

# from fk.api.shopify.Database import *
import pprint
import time
import random
import json

logger = logging.getLogger(__name__)

# fmt:off
product_fields = [
    "id",
    "myshopify_name",
    "handle",
    "body_html",
    "options",
    "product_type",
    "published_at",
    "published_scope",
    "tags",
    "template_suffix",
    "title",
    "variants",
    "vendor",
    "json"]

product_variants_fields = [
    "id",
    "myshopify_name",
    "variant_id",
    "title",
    "quantity",
    "sku",
    "variant_title",
    "vendor",
    "fulfillment_service",
    "product_id",
    "requires_shipping",
    "taxable",
    "gift_card",
    "name",
    "variant_inventory_management",
    "product_exists",
    "fulfillable_quantity",
    "grams",
    "price",
    "total_discount",
    "fulfillment_status",
    "admin_graphql_api_id",
    "json"]
# fmt:on

product_dict_default = {}
product_variants_dict_default = {}

for f in product_fields:
    product_dict_default[f] = None

for f in product_variants_fields:
    product_variants_dict_default[f] = None


def put_batch_job_fetch_product_variant(config, shop_domain, product_id, product_variant_id):
    id = False
    try:
        bp = fk.batch.BatchProcessor.BatchProcessor(config)
        id = bp.insert_batch_item(type="shopify-product-variants-fetch", data=json.dumps({"shop_domain": shop_domain, "product_id": product_id, "product_variant_id": product_variant_id}), priority=50, source=None) or False
    except Exception as e:
        logger.error(f"Could not submit batch job fetch product {product_id} variant {product_variant_id}: {e}", exc_info=True)
    return id


def batch_filter_entrypoint(batch_item={}, config={}):
    # logger.info(f"SHOPIFY PRODUCTS FETCH RUNNING WITH {batch_item} ######################")
    data = json.loads(batch_item.get("data", {}))
    shop_domain = data.get("shop_domain")
    product_id = data.get("product_id")
    if not shop_domain:
        return None, "No shop_domain"
    if not product_id:
        return None, "No product_id"
    shop_domain = shop_domain.strip()
    db = Database(config)  # No flask context: get_flask_shopify_db(config)
    shop = db.get_shop(shop_domain)
    if not shop:
        return None, "No shop"
    token = shop.get("offline_access_token", None)
    if not token:
        return None, "No token"
    sm = ShopifySessionManager(config=config, db=db, shop_domain=shop_domain, token=token)
    product = sm.get_product_by_id(product_id)
    # logger.warning(pprint.pformat(product))
    if not product:
        return None, "No product"
    product_dict_raw = product.to_dict()
    product_dict = {**product_dict_default, **product_dict_raw}
    if product_dict["id"] != product_id:
        return None, f"Returned product {product_dict['id']} did not match initial id {product_id}"
    # Augment it with some needed extras
    product_dict["myshopify_name"] = shop_domain
    product_dict["json"] = json.dumps(product_dict, indent=3, sort_keys=True)
    product_dict["options"] = json.dumps(product_dict["options"], indent=3, sort_keys=True)
    product_dict["variants"] = json.dumps(product_dict["variants"], indent=3, sort_keys=True)
    # logger.warning(pprint.pformat(product_dict["variants"]))
    id = db.upsert_product(product_dict)
    if not id:
        return None, "Could not save product"
    if id != product_id:
        return None, f"Saved id {id} was different from original id {product_id}"
    product_variants = json.loads(product_dict.get("variants", "[]"))
    for product_variants_dict_raw in product_variants:
        product_variant_id = product_variants_dict_raw.get("id")
        if not product_variant_id:
            return None, "No product_variant_id"
        job_id = put_batch_job_fetch_product_variant(config, shop_domain, product_id, product_variant_id)
        if not job_id:
            return None, f"Could not put fetch product variant job for {product_id}"
    return None, None



import logging

from fk.api.shopify import ShopifySessionManager
from fk.api.shopify.db import Database, get_flask_shopify_db
import fk.batch.BatchProcessor

# from fk.api.shopify.Database import *
import pprint
import time
import random
import json

logger = logging.getLogger(__name__)

# fmt:off
order_fields = [
    "id",
    "myshopify_name",
    "status",
    "financial_status",
    "fulfillment_status",
    "email",
    "closed_at",
    "number",
    "note",
    "token",
    "gateway",
    "test",
    "total_price",
    "subtotal_price",
    "total_tax",
    "taxes_included",
    "currency",
    "confirmed",
    "total_discounts",
    "total_line_items_price",
    "cart_token",
    "buyer_accepts_marketing",
    "name",
    "referring_site",
    "landing_site",
    "cancelled_at",
    "cancel_reason",
    "total_price_usd",
    "checkout_token",
    "reference",
    "user_id",
    "location_id",
    "source_identifier",
    "source_url",
    "processed_at",
    "device_id",
    "phone",
    "customer_locale",
    "app_id",
    "browser_ip",
    "landing_site_ref",
    "order_number",
    "processing_method",
    "checkout_id",
    "source_name",
    "tags",
    "contact_email",
    "order_status_url",
    "presentment_currency",
    "json"]


order_line_fields = [
    "id",
    "myshopify_name",
    "variant_id",
    "title",
    "quantity",
    "sku",
    "variant_title",
    "vendor",
    "fulfillment_service",
    "product_id",
    "requires_shipping",
    "taxable",
    "gift_card",
    "name",
    "variant_inventory_management",
    "product_exists",
    "",
    "fulfillable_quantity",
    "grams",
    "price",
    "total_discount",
    "fulfillment_status",
    "admin_graphql_api_id",
    "json"]


product_fields = [
    "id",
    "handle",
    "body_html",
    "options",
    "product_type",
    "published_at",
    "published_scope",
    "tags",
    "template_suffix",
    "title",
    "variants",
    "vendor",
    "json"]
# fmt:on

order_dict_default = {}
order_line_dict_default = {}
product_dict_default = {}

for f in order_fields:
    order_dict_default[f] = None

for f in order_line_fields:
    order_line_dict_default[f] = None

for f in product_fields:
    product_dict_default[f] = None


def images_to_image_src(images):
    if not images or len(images) < 1:
        logger.warning("Error in retrieved product images")
        return "https://fk.z5.no/static/images/favicon.png"
    image = images.pop()
    if not image or not image.src:
        logger.warning("Error in retrieved product image")
        return "https://fk.z5.no/static/images/favicon.png"
    return image.src


def salespop_order_from_order(order_dict, order_line_dict, product):
    customer = order_dict.get("customer", {})
    default_address = customer.get("default_address", {})

    product_url = f"https://{order_dict.get('myshopify_name')}/products/{product.handle}"
    image_src = images_to_image_src(product.images)

    salespop_order = {"order_id": order_dict.get("id"), "order_line_id": order_line_dict.get("id"), "myshopify_name": order_dict.get("myshopify_name"), "product_title": order_line_dict.get("title"), "product_quantity": order_line_dict.get("quantity"), "product_variant_title": order_line_dict.get("variant_title"), "product_vendor": order_line_dict.get("vendor"), "product_price": order_line_dict.get("price"), "product_total_discount": order_line_dict.get("total_discount"), "product_image_url": image_src, "product_url": product_url, "customer_first_name": default_address.get("first_name"), "customer_last_name": default_address.get("last_name"), "customer_city": default_address.get("city"), "customer_country": default_address.get("country"), "customer_country_code": default_address.get("country_code"), "closed_at": order_dict.get("created_at")}
    return salespop_order


def put_batch_job_fetch_product(config, shop_domain, product_id):
    id = False
    if not shop_domain:
        raise Exception("No shop_doamin")
    try:
        bp = fk.batch.BatchProcessor.BatchProcessor(config)
        id = bp.insert_batch_item(type="shopify-products-fetch", data=json.dumps({"shop_domain": shop_domain, "product_id": product_id}), priority=50, source=None) or False
    except Exception as e:
        logger.error(f"Could not submit batch job to fetch product {product_id}: {e}", exc_info=True)
    return id


def batch_filter_entrypoint(batch_item={}, config={}):
    # logger.info(f"SHOPIFY ORDERS FETCH RUNNING WITH {batch_item} ######################")
    data = json.loads(batch_item.get("data", {}))
    if not data:
        return None, "No data"
    shop_domain = data.get("shop_domain")
    if not shop_domain:
        return None, "No shop domain"
    order_id = data.get("order_id")
    if not order_id:
        return None, "No order_id"
    shop_domain = shop_domain.strip()
    db = Database(config)  # No flask context: get_flask_shopify_db(config)
    shop = db.get_shop(shop_domain)
    if not shop:
        return None, "No shop"
    token = shop.get("offline_access_token", None)
    if not token:
        return None, "No token"
    # logger.info(f"TOKEN FROM DB WAS: {token}")
    sm = ShopifySessionManager(config=config, db=db, shop_domain=shop_domain, token=token)
    # sm.self_install()
    order_dict_raw = sm.get_order_by_id(order_id)
    # logger.warning(pprint.pformat(orders))
    if not order_dict_raw:
        return None, "No order"
    # logger.info(f" + ORDER: {order}")
    order_dict = {**order_dict_default, **order_dict_raw}
    # Augment it with some needed extras
    order_dict["myshopify_name"] = shop_domain
    order_dict["json"] = json.dumps(order_dict, indent=3, sort_keys=True)
    # j=json.dumps(order_dict, indent=3, sort_keys=True)
    order_id = db.upsert_order(order_dict)
    if not order_id:
        return None, "Could not upsert order"
    order_lines = order_dict.get("line_items", [])
    for order_line_dict_raw in order_lines:
        order_line_dict = {**order_line_dict_default, **order_line_dict_raw}
        order_line_dict["order_id"] = order_id
        order_line_dict["myshopify_name"] = shop_domain
        order_line_dict["json"] = json.dumps(order_line_dict, indent=3, sort_keys=True)
        # Just put intermediate version of the product here to ensure our foreign key constraint will be met
        product_id = order_line_dict["product_id"]
        product_dict = {**product_dict_default}
        product_dict["id"] = product_id
        product_dict["myshopify_name"] = shop_domain
        # , **{"id": product_id, "myshopify_name": shop_domain}}
        id = db.upsert_product(product_dict)
        if product_id != id:
            logger.warning(f"Could not insert preliminary product for order line with product_id {product_id}, skipping order line...")
            continue
        order_line_id = db.upsert_order_line(order_line_dict)
        if not order_line_id:
            return None, "Could not save order line"
        job_id = put_batch_job_fetch_product(config, shop_domain, product_id)
        if not job_id:
            return None, f"Could not put fetch product job for {product_id}"
        # product_variants_dict["inventory_quantity"] = int(float(product_dict.get("inventory_quantity", "0")))
        # db.upsert_product_variants(product_variants_dict)
        # product = sm.get_product_by_id(product_variants_dict["product_id"])

        # Phasint out the simplified table for now
        # salespop_order_dict = salespop_order_from_order(order_dict, order_line_dict, product)
        # db.upsert_salespop_order(salespop_order_dict)
        # db.update_shop_orders_updated_at(shop_domain)
    return None, None



import logging

from fk.api.shopify.db import Database as ShopifyDB

import fk.batch.BatchProcessor
import datetime

logger = logging.getLogger(__name__)


def batch_filter_entrypoint(batch_item={}, config={}):
    # logger.info(f"SHOPIFY ORDERS FETCH ALL RUNNING WITH {batch_item} ######################")
    db = ShopifyDB(config)
    shops = db.get_shop_names()
    if not shops:
        logger.error(f"No shops")
        return None, None
    bp = fk.batch.BatchProcessor.BatchProcessor(config)
    min_mean = datetime.timedelta(hours=1)
    for shop in shops:
        shop_name = shop.get("myshopify_name", None)
        if not shop_name:
            logger.error(f"No shop name, skipping")
            continue
        count = shop.get("count", 0)
        if not count:
            # logger.warning(f"Count was <1 for shop {shop_name}, skipping")
            continue
        # Calculate if shop is active enough to warrant a new fetch
        interval = shop.get("interval", datetime.timedelta(days=0))
        if not interval:
            logger.error(f"Invalid interval: '{interval}' for shop {shop_name}, skipping")
            continue
        mean_time = shop.get("interval_per_count", datetime.timedelta(days=0))
        if not mean_time:
            logger.error(f"Invalid mean_time: '{mean_time}' for shop {shop_name}, skipping")
            continue
        # count = shop.get("count", 0) logger.info(f"## SHOP: {shop} ct={count} iv={interval} mean={mean_time} min={min_mean}")
        if mean_time < min_mean or interval > min_mean:
            # logger.info("---------------------------------------------------------------")
            try:
                id = bp.insert_batch_item(type="shopify-orders-fetch", data=shop_name, priority=50, source=None) or -1
            except Exception as e:
                logger.error(f"Could not submit batch item", exc_info=True)
    return None, None



import logging

# from flask import url_for

from fk.api.shopify import ShopifySessionManager
from fk.api.shopify.db import Database

import json

logger = logging.getLogger(__name__)


def batch_filter_entrypoint(batch_item={}, config={}):
    # logger.info(f"SHOPIFY PRODUCTS FETCH RUNNING WITH {batch_item} ######################")
    data = json.loads(batch_item.get("data", {}))
    shop_domain = data.get("shop_domain")
    shop_domain = shop_domain.strip()
    if not shop_domain:
        return None, "No shop_domain"
    db = Database(config)  # No flask context: get_flask_shopify_db(config)
    sm = ShopifySessionManager(config=config, db=db, shop_domain=shop_domain)
    si_ok, si_msg = sm.self_install()
    if not si_ok:
        return None, f"Could not self install session manager: {si_msg}"

    # shop = db.get_shop(shop_domain)
    # if not shop:
    #    return None, "No shop"
    # token = shop.get("offline_access_token", None)
    # if not token:
    #    return None, "No token"
    # sm = ShopifySessionManager(config=config, db=db, shop_domain=shop_domain, token=token)
    auth_ok, auth_error = sm.is_authenticated()
    if not auth_ok:
        return None, f"Could not authenticate: {auth_error}"
    script_tag_url = "https://test.merchbot.net/shopify/script_tag"
    # url_for("shopify_spop_bp.script_tag", _external=True)
    logger.info(f"Using script_tag_url='{script_tag_url}'")
    script_tags = sm.get_script_tags(script_tag_url)
    return f"RES: {script_tags}", None


"""
Error creating session with api_url=https://bonkers-test-store.myshopify.com/admin/api/2020-04,
token='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',
shop_domain=bonkers-test-store.myshopify.com,
api-version=2020-04:
    
Response(code=401,
body="b'{"errors":"[API] Invalid API key or access token (unrecognized login or wrong password)"}'", headers={'Date': 'Sun, 27 Sep 2020 23:28:06 GMT', 'Content-Type': 'application/json; charset=utf-8', 'Transfer-Encoding': 'chunked', 'Connection': 'close', 'Set-Cookie': '__cfduid=da3693a61939439155461ca47f9e418f21601249285; expires=Tue, 27-Oct-20 23:28:05 GMT; path=/; domain=.myshopify.com; HttpOnly; SameSite=Lax', 'X-Sorting-Hat-PodId': '34', 'X-Sorting-Hat-ShopId': '5263130659', 'Referrer-Policy': 'origin-when-cross-origin', 'X-Frame-Options': 'DENY', 'X-ShopId': '5263130659', 'X-ShardId': '34', 'WWW-Authenticate': 'Basic Realm="Shopify API Authentication"', 'Strict-Transport-Security': 'max-age=7889238', 'X-Shopify-Stage': 'production', 'Content-Security-Policy': "default-src 'self' data: blob: 'unsafe-inline' 'unsafe-eval' https://* shopify-pos://*; block-all-mixed-content; child-src 'self' https://* shopify-pos://*; connect-src 'self' wss://* https://*; frame-ancestors 'none'; img-src 'self' data: blob: https:; script-src https://cdn.shopify.com https://cdn.shopifycdn.net https://checkout.us.shopifycs.com https://js-agent.newrelic.com https://bam.nr-data.net https://api.stripe.com https://mpsnare.iesnare.com https://appcenter.intuit.com https://www.paypal.com https://js.braintreegateway.com https://c.paypal.com https://maps.googleapis.com https://www.google-analytics.com https://v.shopify.com https://widget.intercom.io https://js.intercomcdn.com 'self' 'unsafe-inline' 'unsafe-eval'; upgrade-insecure-requests; report-uri /csp-report?source%5Baction%5D=show&source%5Bapp%5D=Shopify&source%5Bcontroller%5D=admin%2Fshops&source%5Bsection%5D=admin_api&source%5Buuid%5D=92d591f5-b7a3-49f0-8ac0-e603429b167c", 'X-Content-Type-Options': 'nosniff', 'X-Download-Options': 'noopen', 'X-Permitted-Cross-Domain-Policies': 'none', 'X-XSS-Protection': '1; mode=block; report=/xss-report?source%5Baction%5D=show&source%5Bapp%5D=Shopify&source%5Bcontroller%5D=admin%2Fshops&source%5Bsection%5D=admin_api&source%5Buuid%5D=92d591f5-b7a3-49f0-8ac0-e603429b167c', 'X-Dc': 'gcp-us-central1,gcp-us-east1,gcp-us-east1', 'X-Request-ID': '92d591f5-b7a3-49f0-8ac0-e603429b167c', 'CF-Cache-Status': 'DYNAMIC', 'cf-request-id': '05737e2ed30000ec7a8dbf9200000001', 'Expect-CT': 'max-age=604800, report-uri="https://report-uri.cloudflare.com/cdn-cgi/beacon/expect-ct"', 'Server': 'cloudflare', 'CF-RAY': '5d98ffc48e6cec7a-DFW', 'alt-svc': 'h3-27=":443"; ma=86400, h3-28=":443"; ma=86400, h3-29=":443"; ma=86400'}, msg="Unauthorized")
2020-09-27 23:28:06 INFO (MainThread) [pyactiveresource.connection:257::_open()] - GET https://bonkers-test-store.myshopify.com/admin/api/2020-04/products/123.json
"""
